package cn.jlsxwkj.common.handler;

import cn.dev33.satoken.exception.SaTokenException;
import cn.jlsxwkj.common.R.ResponseError;
import cn.jlsxwkj.common.exception.CustomException;
import cn.jlsxwkj.common.exception.InsertFailException;
import cn.jlsxwkj.common.utils.Log;
import cn.jlsxwkj.moudles.logerror.LogError;
import cn.jlsxwkj.moudles.logerror.LogErrorService;
import jakarta.annotation.Resource;
import org.springframework.web.ErrorResponse;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.Arrays;

/**
 * @author zh
 */
@RestControllerAdvice
public class GlobalRestExceptionHandler {

    @Resource
    private LogErrorService logErrorService;

    /**
     * 捕捉全局异常
     *
     * @param e 异常
     * @return 封装异常对象
     */
    @ExceptionHandler(Throwable.class)
    public ResponseError exception(Exception e) {

        ResponseError responseError = ResponseError.data(e.getClass().getName());

        if (e instanceof CustomException) {
            responseError.setMessage(e.getMessage());
        }
        if (e instanceof ErrorResponse) {
            responseError.setCode(((ErrorResponse) e).getStatusCode().value());
            responseError.setMessage(e.getMessage());
        }
        if (e instanceof SaTokenException) {
            responseError.setCode(((SaTokenException) e).getCode());
            responseError.setMessage("账户未登录");
        }

        LogError errorHandlerToLogError = new LogError().castResponseErrorToLogError(responseError);
        errorHandlerToLogError.setErrorInfo(e.getMessage());
        errorHandlerToLogError.setErrorStackTrace(Arrays.toString(e.getStackTrace()));
        try {
            logErrorService.insertOne(errorHandlerToLogError);
        } catch (InsertFailException insertFailException) {
            Log.warn(this.getClass(), insertFailException.getMessage());
        }

        Log.error(e.getClass(), "Code       ====> {}", responseError.getCode());
        Log.error(e.getClass(), "Exception  ====> {}", responseError.getException());
        Log.error(e.getClass(), "Message    ====> {}", responseError.getMessage());

        e.printStackTrace();

        return responseError;
    }
}
