package cn.jlsxwkj.moudles.userlist;

import cn.dev33.satoken.secure.SaSecureUtil;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.lang.Snowflake;
import cn.jlsxwkj.common.exception.*;
import cn.jlsxwkj.common.utils.CheckPassword;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author zh
 */
@Service
public class UserListService {

    @Resource
    private UserListMapper userListMapper;
    private final Snowflake snowflake = new Snowflake();

    @Transactional(rollbackFor = AccountAuthFailException.class)
    public UserVO checkOrAddUser(String userName, String userPassword) throws CustomException {
        if (!CheckPassword.checkPassword(userPassword)) {
            throw new AccountPasswordCheckFailException("密码长度为8到20位,必须包含字母和数字，字母区分大小写");
        }
        String sha512pwd = SaSecureUtil.sha512(userPassword);
        UserList userList = userListMapper.checkUser(userName);
        UserVO userVO = new UserVO();
        String userId;
        if (userList != null) {
            if (!userList.getUserPassword().contains(sha512pwd)) {
                throw new LoginWrongPasswordException("密码错误");
            }
            if (!StpUtil.isLogin()) {
                userId = userList.getUserId();
                StpUtil.login(userId, false);
                userVO.setUserId(userId);
                userVO.setUserName(userList.getUserName());
                userVO.setMessage("登录成功");
                return userVO;
            }
            throw new LoginAccountAlreadyLoginException("请勿重复登录");
        }
        userList = new UserList();
        userList.setUserId(snowflake.nextIdStr());
        userList.setUserName(userName);
        userList.setUserPassword(sha512pwd);
        Integer rows = userListMapper.authUser(userList);
        if (rows > 0) {
            userId = userList.getUserId();
            StpUtil.login(userId, false);
            userVO.setUserId(userId);
            userVO.setUserName(userList.getUserName());
            userVO.setMessage("注册成功");
            return userVO;

        }
        throw new AccountAuthFailException("注册失败");
    }
}
